local DepthFirst = require( 'Scripts.Depth-First' )

function love.load()	
	love.graphics.setBackgroundColor( 255, 255, 255 )
	love.math.noise( os.time() ) 
	
	Timer = .001
	Time = 0
	ShouldContinue = true
	
	Quads, Maze, QuadWidth, QuadHeight = LoadImageInformation()
	Width, Height = 80, 60
	
	ScreenWidth, ScreenHeight = love.graphics.getWidth(), love.graphics.getHeight()
	ScaleX, ScaleY = ScreenWidth / ( Width * QuadWidth ), ScreenHeight / ( Height * QuadHeight )
	
	-- Mazes
	StartX, StartY = love.math.random( Width ), love.math.random( Height )
	x, y, DepthMaze, VisitedCells, ReverseAmount = DepthFirst.NewStep( Width, Height, StartX, StartY )
	EndX, EndY = nil, nil
	
	-- References
	DepthReference = DepthFirst.DecodeMazeData( DepthMaze )
	
	-- Canvases
	love.graphics.setDefaultFilter( 'nearest', 'nearest' )
	DepthCanvas = SetToCanvas( DepthReference, Quads, Maze, QuadWidth, QuadHeight )
end

function love.draw()
	love.graphics.push()
		love.graphics.scale( ScaleX, ScaleY )				
		
		love.graphics.setColor( 255, 0, 0, 255 )
		if ShouldContinue or EndX then
			love.graphics.rectangle( 'fill', ( x - 1 ) * QuadWidth, ( y - 1 ) * QuadHeight, QuadWidth, QuadHeight )
		end
		
		love.graphics.setColor( 0, 255, 0, 255 )
		love.graphics.rectangle( 'fill', ( StartX - 1 ) * QuadWidth, ( StartY - 1 ) * QuadHeight, QuadWidth, QuadHeight )
		
		love.graphics.setColor( 255, 255, 255, 255 )
		
		love.graphics.draw( DepthCanvas, 0, 0 )
		
		love.graphics.setColor( 255, 0, 0, 255 )
		-- love.graphics.print( math.floor( love.mouse.getX() / ( QuadWidth * ScaleX ) ) + 1 .. ', ' .. math.floor( love.mouse.getY() / ( QuadHeight * ScaleY ) ) + 1, 1, 1 )
	love.graphics.pop()
end

function love.update( dt ) 
	Time = Time + dt
	if ShouldContinue and Time > Timer then
		for a = 1, 1 do -- Using this, you can set how many times you want to call per frame- to speed up the process.
			if not DepthReference[y] or not DepthReference[y][x] or not ReverseAmount then End() return end
			
			x, y, ReverseAmount = DepthFirst.Step( x, y, DepthMaze, VisitedCells, ReverseAmount )
			DepthReference = DepthFirst.DecodeMazeData( DepthMaze )
		end
		
		DepthCanvas = SetToCanvas( DepthReference, Quads, Maze, QuadWidth, QuadHeight )
		Time = 0
	end 
end

function love.keypressed( Key, IsRepeat )
	if Key == 'escape' then love.event.quit() end
	
	if DepthFirst.UnvisitedCellsRemain( DepthMaze ) then 
		ShouldContinue = not ShouldContinue 
		if not ShouldContinue then love.window.setTitle( 'Paused' ) 
		else love.window.setTitle( 'Maze Creation Algorithms' ) end
	else x, y = DepthFirst.ChooseRandomEndCell( DepthReference ) end
end

function love.keyreleased( Key, IsRepeat )
end

function End()
	-- TakeScreenShot() 
	-- PrintMazeData( DepthReference )
	-- local LastCell = VisitedCells[#VisitedCells]
	-- print( LastCell.x, LastCell.y )
	
	DepthCanvas = SetToCanvas( DepthReference, Quads, Maze, QuadWidth, QuadHeight )
	ShouldContinue = false 
	love.window.setTitle( 'Maze Creation Algorithms' )
	EndX, EndY = DepthFirst.ChooseRandomEndCell( DepthReference )
	x, y = EndX, EndY
end

function TakeScreenShot()
	local Image = love.graphics.newScreenshot()
	love.filesystem.write( 'Maze.png', Image )
	Image:encode( 'Maze.png' )
end

function MakePO2( Dimension )
	local a = 1
	while a < Dimension do
		a = a * 2
	end
	return a
end

function SetToCanvas( Reference, Quads, Maze, QuadWidth, QuadHeight )
	local Canvas = love.graphics.newCanvas( MakePO2( ScreenWidth ), MakePO2( ScreenHeight ) )
	
	love.graphics.setCanvas( Canvas )
		Canvas:clear()
		
		for y = 1, #Reference do
			for x = 1, #Reference[y] do				
				love.graphics.draw( Maze, Quads[ Reference[y][x] ], ( x - 1 ) * QuadWidth, ( y - 1 ) * QuadHeight )
			end
		end
	love.graphics.setCanvas()
	
	return Canvas
end

function PrintMazeData( Reference )
	for y = 1, #Reference do
		for x = 1, #Reference[y] do
			if #Reference[y][x] < 2 then print( x, y, Reference[y][x] ) end
		end
	end
end

function LoadImageInformation()
	local Quads = {}
	
	local Maze = love.graphics.newImage( 'Images/Maze.png' )
	local QuadWidth, QuadHeight = 5, 5
	local QuadInformation = {
		L = { 0, 0 }, 
		LR = { 5, 0 }, 
		LU = { 10, 0 }, 
		LD = { 15, 0 }, 
		LRU = { 20, 0 }, 
		LRD = { 25, 0 }, 
		LUD = { 0, 5 }, 
		LRUD = { 5, 5 }, 
		R = { 10, 5 }, 
		U = { 15, 5 }, 
		D = { 20, 5 },
		RD = { 25, 5 }, 
		UD = { 0, 10 }, 
		RU = { 5, 10 }, 
		RUD = { 10, 10 }, 
	}
	QuadInformation[''] = { 15, 10 }
	for Index, Quad in pairs( QuadInformation ) do
		Quads[Index] = love.graphics.newQuad( QuadInformation[Index][1], QuadInformation[Index][2], QuadWidth, QuadHeight, Maze:getWidth(), Maze:getHeight() )
	end
	
	return Quads, Maze, QuadWidth, QuadHeight
end